package objects

import "git.tebibyte.media/tomo/tomo"
import "git.tebibyte.media/tomo/tomo/input"
import "git.tebibyte.media/tomo/tomo/event"

// Checkbox is a control that can be toggled.
type Checkbox struct {
	tomo.Box
	value bool
	on struct {
		valueChange event.FuncBroadcaster
	}
}

// NewCheckbox creates a new checkbox with the specified value.
func NewCheckbox (value bool) *Checkbox {
	box := &Checkbox {
		Box: tomo.NewBox(),
	}
	box.SetRole(tomo.R("objects", "Checkbox"))
	box.SetValue(value)
	
	box.OnButtonDown(box.handleButtonDown)
	box.OnButtonUp(box.handleButtonUp)
	box.OnKeyDown(box.handleKeyDown)
	box.OnKeyUp(box.handleKeyUp)
	box.SetFocusable(true)
	return box
}

// Value returns the value of the checkbox.
func (this *Checkbox) Value () bool {
	return this.value
}

// SetValue sets the value of the checkbox.
func (this *Checkbox) SetValue (value bool) {
	this.value = value
	// the theme shall decide what checked and unchecked states look like
	this.SetTag("checked",    value)
	this.SetTag("unchecked", !value)
}

// Toggle toggles the value of the checkbox between true and false.
func (this *Checkbox) Toggle () {
	this.SetValue(!this.Value())
}

// OnValueChange specifies a function to be called when the user checks or
// unchecks the checkbox.
func (this *Checkbox) OnValueChange (callback func ()) event.Cookie {
	return this.on.valueChange.Connect(callback)
}

func (this *Checkbox) handleKeyDown (key input.Key, numberPad bool) bool {
	if key != input.KeyEnter && key != input.Key(' ') { return false }
	this.Toggle()
	return true
}

func (this *Checkbox) handleKeyUp (key input.Key, numberPad bool) bool {
	if key != input.KeyEnter && key != input.Key(' ') { return false}
	return true
}

func (this *Checkbox) handleButtonDown (button input.Button) bool {
	if button != input.ButtonLeft { return false }
	return true
}

func (this *Checkbox) handleButtonUp (button input.Button) bool {
	if button != input.ButtonLeft { return false }
	if this.Window().MousePosition().In(this.Bounds()) {
		this.Toggle()
	}
	return true
}
